var _this = this;

/*
 * @Description: 公共工具类
 * @Author: SmartUI深圳开发团队
 */

/**
 * 防抖函数
 * @param method 事件触发的操作
 * @param delay 多少毫秒内连续触发事件，不会执行
 * @returns {Function}
 */
export const asp_Debounce = (func, wait, immediate) => {
  let timeout, args, context, timestamp, result;

  const later = () => {
    // 据上一次触发时间间隔
    const last = +new Date() - timestamp;

    // 上次被包装函数被调用时间间隔 last 小于设定时间间隔 wait
    if (last < wait && last > 0) {
      timeout = setTimeout(later, wait - last);
    } else {
      timeout = null;
      // 如果设定为immediate===true，因为开始边界已经调用过了此处无需调用
      if (!immediate) {
        result = func.apply(context, args);
        if (!timeout) context = args = null;
      }
    }
  };

  return (...args) => {
    context = _this;
    timestamp = +new Date();
    const callNow = immediate && !timeout;
    // 如果延时不存在，重新设定延时
    if (!timeout) timeout = setTimeout(later, wait);
    if (callNow) {
      result = func.apply(context, args);
      context = args = null;
    }

    return result;
  };
};

/**
 * 深拷贝
 */
export function asp_DeepClone(source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'deepClone');
  }
  const targetObj = source.constructor === Array ? [] : {};
  Object.keys(source).forEach(keys => {
    if (source[keys] && typeof source[keys] === 'object') {
      targetObj[keys] = asp_DeepClone(source[keys]);
    } else {
      targetObj[keys] = source[keys];
    }
  });
  return targetObj;
}

/**
 * 区间 随机数
 * @param {*} min  最小值
 * @param {*} max  最大值
 */
export const asp_RandomNumber = (min, max) => {
  return Math.floor(Math.random() * (max - min + 1)) + min;
};

/**
 * 生成随机数字符串，指定字符串的长度，同时规定每一位的大小范围，并且指定每一个数的显示基数
 * @param {*} min  最小值
 * @param {*} max  最大值
 * @param {*} bitNumber  随机数包括几位
 * @param {*} radix  每一位的基数, 2-数字以二进制值显示; 8-数字以八进制值显示; 10-数字以十进制值显示; 16-数字以十六进制值显示
 */
export const asp_randomNumString = (min, max, bitNumber, radix) => {
  var result = '';
  for (var i = 0; i < bitNumber; i++) {
    result += asp_RandomNumber(min, max).toString(radix);
  }
  // 默认字母小写，手动转大写
  return result.toUpperCase();
};

/**
 * 生成唯一标识符UUID
 */
export const asp_generateUUID = () => {
  let d = new Date().getTime();
  if (window.performance && typeof window.performance.now === 'function') {
    d += performance.now(); // use high-precision timer if available
  }
  const uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    const r = (d + Math.random() * 16) % 16 | 0;
    d = Math.floor(d / 16);
    return (c === 'x' ? r : r & 0x3 | 0x8).toString(16);
  });
  return uuid;
};

export default {
  asp_Debounce,
  asp_DeepClone,
  asp_RandomNumber,
  asp_generateUUID
};